#include "glut_import.h"

/*********************************************************************
 * GLOBAL DATA
 *********************************************************************/

struct {
  int Theta1,Theta2,Theta3,Theta4,Theta5;
} Data={
  0,0,0,0,0
};

/*********************************************************************
 * GLUT CALLBACKS
 *********************************************************************/

void reshapeFunc(int width,int height){
  glViewport(0,0,width,height);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  /* Vue en perspective de mme ratio (width/height) que la fentre */
  gluPerspective(70,(float) width/height,1,20);
}

/* Cette fonction affiche un pav de taille length x diameter x diameter
   parallle aux axes du repre courant, dont l'extrmit gauche est dispose
   sur l'origine du repre courant, et dplace ensuite l'origine sur l'extrmit
   droite du pav. */
void tube(float length,float diameter){
  /* On sauvegarde la matrice du repre courant */
  glPushMatrix();
  /* Translation/dilatation pour placer les bornes du pav au bon endroit */
  glScalef(length,diameter,diameter);
  glTranslatef(0.5,0,0);
  /* Affichage du pav */
  glutSolidCube(1);
  /* On revient  la matrice de dpart */
  glPopMatrix();
  /* On dplace l'origine sur l'extrmit droite du pav */
  glTranslatef(length,0,0);
}

void displayFunc(void){
  /* On efface */
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  /* On rinitialise la matrice */
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  /* On place la camra */
  gluLookAt(0,0,10,0,0,0,0,1,0);

  /* La rotation d'angle Theta1 est applique  la scne en entier */
  glRotatef(Data.Theta1,0,1,0);
  /* Rotation d'angle Theta2 est ensuite applique  la scne en entier */
  glRotatef(Data.Theta2,1,0,0);

  /* On place notre origine locale  l'extrmit gauche du bras (-7,0,0) */
  glTranslatef(-7,0,0);

  /* 1er segment du bras */
  tube(4,0.6);
  /* Sphre au bout du 1er segment */
  glutSolidSphere(0.5,100,100);

  /* Rotation du 2me segment par rapport  l'origine locale, c'est  dire
     l'extrmit droite du 1er segment */
  glRotatef(Data.Theta3,0,0,1);
  /* 2me segment du bras */
  tube(4,0.5);
  /* Sphre au bout du 2me segment */
  glutSolidSphere(0.4,100,100);

  /* Rotation du 3me segment par rapport  l'origine locale, c'est  dire
     l'extrmit droite du 2me segment */
  glRotatef(Data.Theta4,0,0,1);
  /* 3me segment du bras */
  tube(4,0.4);
  /* Sphre au bout du 3me segment */
  glutSolidSphere(0.3,100,100);

  /* On sauvegarde la matrice avant de commencer la premire branche de la pince */
  glPushMatrix();
  /* Angle du premier segment de la branche */
  glRotatef(Data.Theta5+30,0,0,1);
  /* Premier segment de la premire branche */
  tube(1.5,0.2);
  /* Sphre au bout du premier segment */
  glutSolidSphere(0.2,100,100);
  /* Angle du second segment de la premire branche */
  glRotatef(-40,0,0,1);
  /* Second segment de la premire branche */
  tube(1,0.2);
  /* On revient  la matrice juste avant la premire branche */
  glPopMatrix();

  /* Angle du premier segment de la seconde branche */
  glRotatef(-Data.Theta5-30,0,0,1);
  /* Premier segment de la seconde branche */
  tube(1.5,0.2);
  /* Sphre au bout du premier segment */
  glutSolidSphere(0.2,100,100);
  /* Angle du second segment de la seconde branche */
  glRotatef(40,0,0,1);
  /* Second segment de la seconde branche */
  tube(1,0.2);

  glFlush();
  glutSwapBuffers();
}

void keyboardFunc(unsigned char key,int x,int y){
  switch (key){
    case '0':
      Data.Theta1++;
      break;
    case '1':
      Data.Theta1--;
      break;
    case '2':
      Data.Theta2++;
      break;
    case '3':
      Data.Theta2--;
      break;
    case '4':
      Data.Theta3++;
      break;
    case '5':
      Data.Theta3--;
      break;
    case '6':
      Data.Theta4++;
      break;
    case '7':
      Data.Theta4--;
      break;
    case '8':
      Data.Theta5++;
      break;
    case '9':
      Data.Theta5--;
      break;
    case 'Q':
    case 'q':
      exit(0);
  }
  glutPostRedisplay();
}

/*********************************************************************
 * MAIN PROGRAM
 *********************************************************************/

int main(int argc,char *argv[]){
  int a=800,b=600,x,y;
  glutInit(&argc,argv);
  x=glutGet(GLUT_SCREEN_WIDTH);
  y=glutGet(GLUT_SCREEN_HEIGHT);
  glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH);
  glutInitWindowPosition((x-a)/2,(y-b)/2);
  glutInitWindowSize(a,b);
  glutCreateWindow("Robot arm");
  glutReshapeFunc(reshapeFunc);
  glutDisplayFunc(displayFunc);
  glutKeyboardFunc(keyboardFunc);
  glEnable(GL_LIGHTING);
  glEnable(GL_COLOR_MATERIAL);
  glEnable(GL_LIGHT0);
  glEnable(GL_DEPTH_TEST);
  glutMainLoop();
}
